"""
Install method

$Id: Install.py 23 2008-05-06 10:01:35Z gael.le-mignot $
"""

from StringIO import StringIO

from Products.CMFCore.utils import getToolByName
try:
    from Products.CMFDynamicViewFTI.migrate import migrateFTIs
    has_migrate = True
except:
    has_migrate = False

from Products.Archetypes.public import listTypes, process_types
from Products.Archetypes.Extensions.utils import installTypes, install_subskin

from Products.SignableEvent.config import PROJECT_NAME, GLOBALS, SKINS_DIR, CONFIGLETS
from Products.SignableEvent.tools.SignableEventTool import ID, SignableEventTool

from Products.SignableEvent import workflow

def setWorkflowChains(self, meta_types, workflow):
    """Restore a workflow
    """
    wftool = getToolByName(self, 'portal_workflow')
    if "default_chain" in meta_types:
        wftool.setDefaultChain(workflow)
        meta_types = [ m for m in meta_types if m != "default_chain" ]
    wftool.setChainForPortalTypes(meta_types, workflow)

def installWorkflow(self, workflow_id, workflow, meta_types):
    """
    Install a workflow
    
    if meta_types contains 'default_chain' it will also be set as the default workflow 
    """
    wftool = getToolByName(self, 'portal_workflow')
    workflow = workflow.create_workflow(workflow_id)
    wftool._setObject(workflow_id, workflow)
    setWorkflowChains(self, meta_types, workflow_id)
    
def install_configlet(self, out):
    configTool = getToolByName(self, 'portal_controlpanel', None)
    if configTool:
        for conf in CONFIGLETS:
            if not configTool.getActionObject("Products/%s" % conf['id']):
                try:
                    configTool.registerConfiglet(**conf)
                except KeyError:
                    pass


def uninstall_configlet(self,out):
    configTool = getToolByName(self, 'portal_controlpanel', None)
    if configTool:
        for conf in CONFIGLETS:
            if configTool.getActionObject("Products/%s" % conf['id']):
                try:
                    configTool.unregisterConfiglet(conf['id'])
                except KeyError:
                    pass
    
def install(self):
    """
    Install this product
    """
    out = StringIO()
    print >> out, 'Installing %s' % PROJECT_NAME

    ## Install skin
    install_subskin(self, out, GLOBALS, SKINS_DIR)
    print >> out, 'Installed skin.'

    ## Add tool
    portal = getToolByName(self, 'portal_url').getPortalObject()
    if getToolByName(portal, ID, None) is None:
        t = SignableEventTool()
        portal._setObject(ID, t)
    t = getToolByName(portal, ID)

    ## Install types
    types = listTypes(PROJECT_NAME)
    installTypes(self, out, types, PROJECT_NAME)
    print >> out, 'Installed types.'

    ## Migrate FTIs
    if has_migrate:
        migrated = migrateFTIs(self, PROJECT_NAME)
        print >> out, "Switched to DynamicViewFTI: %s" % ', '.join(migrated)

    ## Enable portal_factory for the new content types
    factory = getToolByName(self, 'portal_factory')
    new_types, useless1, useless2 = process_types(types, PROJECT_NAME)
    types = factory.getFactoryTypes().keys()
    for new_type in new_types:
	if new_type.portal_type not in types:
	    types.append(new_type.portal_type)
    factory.manage_setPortalFactoryTypes(listOfTypeIds=types)
    print >> out, "Added new content types to 'portal_factory'."

    # Register SignableEvent to calendar
    calendar = getToolByName(self, "portal_calendar")
    calendar.calendar_types += ("SignableEvent",)

    # Configure the navigation
    props = self.portal_properties.navtree_properties
    types = list(props.getProperty("metaTypesNotToList", []))
    types_to_exclude = [ "EventSignup" ]
    for typ in types_to_exclude:
        if not typ in types:
            types.append(typ)
    props.manage_changeProperties(metaTypesNotToList = types)

    # Configure the usable page types
    types = list(props.getProperty("default_page_types", []))
    types_to_add = [ "SignableEvent" ]
    for typ in types_to_add:
        if not typ in types:
            types.append(typ)
    props.manage_changeProperties(default_page_types = types)

    # Add actions to content types
    ptypes = getToolByName(self, "portal_types")
    se = ptypes.SignableEvent
    se.addAction(id='signers',
                 name='View signers',
                 action = 'string:${object_url}/signable_event_signers',
                 condition = "",
                 permission = ("List folder contents", ),
                 category = 'object',
                 visible=1) 
    es = ptypes.EventSignup
    edit = es.getActionObject("object/edit")
    edit.edit(action='string:${object_url}/eventsignup_edit')

    
    # Install the workflow
    installWorkflow(self, 'event_signer_workflow',
                    workflow,
                    ('EventSignup', ))


    # Configlets
    install_configlet(self,out)

    print >> out, "Successfully installed %s." % PROJECT_NAME
    return out.getvalue()
    
def uninstall(self):
    out = StringIO()
    print >> out, 'Unnstalling %s' % PROJECT_NAME

    uninstall_configlet(self,out)

    print >> out, "Successfully installed %s." % PROJECT_NAME
    return out.getvalue()
